<?php

use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeForm;
use Drupal\node\NodeInterface;
use Drupal\node\NodeTypeForm;
use Drupal\node\NodeTypeInterface;

/**
 * Implements hook_form_BASE_FORM_ID_alter(): node_type_form.
 */
function improvements_form_node_type_form_alter(array &$form, FormStateInterface $form_state, string $form_id) {
  $node_type_form_object = $form_state->getFormObject(); /** @var NodeTypeForm $node_type_form_object */
  $node_type = $node_type_form_object->getEntity(); /** @var NodeTypeInterface $node_type */

  $form['submission']['hide_revision_group'] = [
    '#type' => 'checkbox',
    '#title' => t('Hide revision group'),
    '#default_value' => $node_type->getThirdPartySetting('improvements', 'hide_revision_group'),
  ];

  $form['submission']['move_status_to_vertical_tab'] = [
    '#type' => 'checkbox',
    '#title' => t('Move "Status" field to vertical tab'),
    '#default_value' => $node_type->getThirdPartySetting('improvements', 'move_status_to_vertical_tab'),
  ];

  if ($node_type->isNew()) {
    $node_type->setThirdPartySetting('menu_ui', 'available_menus', []);

    $form['submission']['preview_mode']['#default_value'] = 0;
    $form['workflow']['options']['#default_value']['promote'] = FALSE;
    $form['workflow']['options']['#default_value']['revision'] = FALSE;
    $form['display']['display_submitted']['#default_value'] = FALSE;
  }

  $form['#entity_builders'][] = 'improvements_node_type_form_entity_builder';
}

/**
 * Node type form entity builder callback.
 */
function improvements_node_type_form_entity_builder(string $entity_type, NodeTypeInterface $node_type, array &$form, FormStateInterface $form_state) {
  $node_type->setThirdPartySetting('improvements', 'hide_revision_group', $form_state->getValue('hide_revision_group'));
  $node_type->setThirdPartySetting('improvements', 'move_status_to_vertical_tab', $form_state->getValue('move_status_to_vertical_tab'));
}

/**
 * Implements hook_form_BASE_FORM_ID_alter(): node_form.
 */
function improvements_form_node_form_alter(array &$form, FormStateInterface $form_state, string $form_id) {
  $node_form_object = $form_state->getFormObject(); /** @var NodeForm $node_form_object */
  $node = $node_form_object->getEntity(); /** @var NodeInterface $node */

  // Add autofocus to title field
  if ($node->isNew()) {
    $form['title']['widget'][0]['value']['#attributes']['autofocus'] = 'autofocus';
  }

  // Hide revision group
  $node_type = $node->get('type')->entity; /** @var NodeTypeInterface $node_type */
  if ($node_type->getThirdPartySetting('improvements', 'hide_revision_group') && !$form['revision']['#default_value']) {
    $form['revision_information']['#access'] = FALSE;
  }

  // Move status field to fieldset
  if ($node_type->getThirdPartySetting('improvements', 'move_status_to_vertical_tab')) {
    $form['status']['#group'] = 'options';
    $form['options']['#title'] = t('Publishing options');
  }
}
