<?php

use Drupal\Core\Render\BubbleableMetadata;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\improvements\Plugin\views\display_extender\ImprovementsDisplayExtender;
use Drupal\improvements\Plugin\views\filter\ExtendedNumericFilter;
use Drupal\views\ViewExecutable;

/**
 * Implements hook_views_plugins_filter_alter().
 */
function improvements_views_plugins_filter_alter(array &$plugins) {
  $plugins['numeric']['class'] = ExtendedNumericFilter::class;
}

/**
 * Implements hook_field_views_data_alter().
 */
function improvements_field_views_data_alter(array &$data, FieldStorageConfigInterface $field_storage) {
  // Add Views contextual filter "with children" for term fields
  if ($field_storage->getType() == 'entity_reference' && $field_storage->getSetting('target_type') == 'taxonomy_term') {
    foreach ($data as $table_name => $table_data) {
      foreach ($table_data as $field_name => $field_data) {
        if (isset($field_data['argument']) && $field_data['argument']['id'] == 'numeric' && $field_name != 'delta') {
          $data[$table_name][$field_name . '_with_children'] = $data[$table_name][$field_name];
          $field = &$data[$table_name][$field_name . '_with_children'];
          $field['title'] .= ' (' . t('With children') . ')';
          $field['argument']['id'] = 'term_with_children';
          unset($field['filter']);
          unset($field['sort']);
        }
      }
    }
  }
}

/**
 * Implements hook_views_data_alter().
 */
function improvements_views_data_alter(array &$data) {
  if (isset($data['node_field_data'])) {
    $data['node_field_data']['uid_is_current_user'] = [
      'title' => t('Author is current user'),
      'title short' => t('Author is current user'),
      'real field' => 'uid',
      'sort' => [
        'id' => 'is_current_user',
      ],
    ];
  }

  if (isset($data['comment_field_data'])) {
    $data['comment_field_data']['uid_is_current_user'] = [
      'title' => t('Author is current user'),
      'title short' => t('Author is current user'),
      'real field' => 'uid',
      'sort' => [
        'id' => 'is_current_user',
      ],
    ];
  }
}

/**
 * @see improvements_token_info()
 */
function improvements_views_token_info() {
  $info = [];

  $info['tokens']['view']['total-rows-in-current-page'] = [
    'name' => t('Total rows in current page'),
    'description' => t('Total rows in current page'),
  ];

  return $info;
}

/**
 * @see improvements_tokens()
 */
function improvements_views_tokens(string $type, array $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {
  $replacements = [];

  // [view:*]
  if ($type == 'view') {
    foreach ($tokens as $name => $original) {
      if ($name == 'total-rows-in-current-page') {
        $view = $data['view']; /** @var ViewExecutable $view */
        $replacements[$original] = count($view->result);
      }
    }
  }

  return $replacements;
}

/**
 * Preprocess function for views-view.html.twig.
 */
function improvements_preprocess_views_view(array &$vars) {
  $view = $vars['view']; /** @var ViewExecutable $view */
  $view_display_extenders = $view->getDisplay()->getExtenders();

  if (!isset($vars['content_attributes'])) {
    $vars['content_attributes'] = [];
  }

  if (isset($view_display_extenders['improvements_display_extender'])) {
    $improvements_display_extender = $view_display_extenders['improvements_display_extender']; /** @var ImprovementsDisplayExtender $improvements_display_extender */
    if (!empty($improvements_display_extender->options['content_css_class'])) {
      $view_content_css_class = $improvements_display_extender->options['content_css_class'];

      // Replace [view:*] tokens
      if (strpos($view_content_css_class, '[view:') !== FALSE) {
        $view_content_css_class = \Drupal::token()->replace($view_content_css_class, [
          'view' => $view,
        ]);
      }

      $vars['content_attributes']['class'][] = $view_content_css_class;
    }
  }
}
