<?php

namespace Drupal\improvements\Cache;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\Context\CacheContextInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\improvements\ImprovementsHelper;

/**
 * Cache context ID: 'parent_term' (to vary by direct parents terms).
 */
class ParentTermCacheContext implements CacheContextInterface {

  /**
   * @var RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Class constructor.
   */
  public function __construct(RouteMatchInterface $route_match) {
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritDoc}
   */
  public static function getLabel() {
    return t('Parent term');
  }

  /**
   * {@inheritDoc}
   */
  public function getContext() {
    if ($current_term_id = $this->routeMatch->getRawParameter('taxonomy_term')) {
      $parents_terms = ImprovementsHelper::getTermDirectParentsIds($current_term_id);
      return $parents_terms ? implode(',', $parents_terms) : (string)$current_term_id;
    }

    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata() {
    return new CacheableMetadata();
  }

}
