<?php

namespace Drupal\improvements;

use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseForControllerResultEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event Subscriber MyEventSubscriber.
 */
class ImprovementsEventSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::VIEW][] = ['onKernelView', 10];
    $events[KernelEvents::RESPONSE][] = ['onKernelResponse', 10];
    return $events;
  }

  /**
   * KernelEvents::VIEW event callback.
   */
  public function onKernelView(GetResponseForControllerResultEvent $event) {
    // Create hook_page_ROUTE_NAME_result_alter()
    $result = $event->getControllerResult();
    $current_route_name = \Drupal::routeMatch()->getRouteName();
    \Drupal::moduleHandler()->alter('page_' . str_replace('.', '_', $current_route_name) . '_result', $result);
    $event->setControllerResult($result);
  }

  /**
   * KernelEvents::RESPONSE event callback.
   */
  public function onKernelResponse(FilterResponseEvent $event) {
    // Remove toolbar/toolbar library from page attachments
    $response = $event->getResponse();
    if (method_exists($response, 'getAttachments')) {
      $attachments = $response->getAttachments();
      if (isset($attachments['library'])) {
        $attachments['library'] = array_diff($attachments['library'], ['toolbar/toolbar']);
        $response->setAttachments($attachments);
      }
    }
  }

}
