<?php

namespace Drupal\improvements;

use Drupal\contact\MessageInterface;
use Drupal\Core\Entity\ContentEntityStorageInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\TermInterface;
use Drupal\taxonomy\TermStorageInterface;

class ImprovementsHelper {

  /**
   * Return term depth.
   */
  public static function getTermDepth(TermInterface $term): int {
    $term_depth = 0;

    while ($term = $term->get('parent')->entity) {
      $term_depth++;
    }

    return $term_depth;
  }

  /**
   * Return term direct children.
   *
   * @return TermInterface[]
   */
  public static function getTermDirectChildren(TermInterface $term): array {
    $term_storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term'); /** @var TermStorageInterface $term_storage */
    return $term_storage->getChildren($term);
  }

  /**
   * Return term direct children count.
   */
  public static function getTermDirectChildrenCount(int $term_id): int {
    return \Drupal::entityQuery('taxonomy_term')
      ->condition('parent', $term_id)
      ->count()
      ->execute();
  }

  /**
   * Return all children terms ids regardless of depth.
   *
   * @return integer[]
   */
  public static function getTermAllChildrenIds(string $vocabulary_machine_name, int $term_id): array {
    $term_storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term'); /** @var TermStorageInterface $term_storage */
    $all_children_terms = $term_storage->loadTree($vocabulary_machine_name, $term_id); /** @var \stdClass $terms */
    $ids = [];

    foreach ($all_children_terms as $children_term) {
      $ids[] = (int)$children_term->tid;
    }

    return $ids;
  }

  /**
   * Return term direct parents ids.
   */
  public static function getTermDirectParentsIds(int $term_id): array {
    $term = Term::load($term_id);
    $parent_ids = [];

    foreach ($term->get('parent') as $item) {
      if ($target_id = $item->target_id) {
        $parent_ids[] = $target_id;
      }
    }

    return $parent_ids;
  }

  /**
   * Return TRUE if term has children terms.
   */
  public static function termHasChildren(int $term_id): bool {
    return (self::getTermDirectChildrenCount($term_id) > 0);
  }

  /**
   * Return contact form.
   */
  public static function getContactForm(string $contact_form_id): array {
    $contact_message_storage = \Drupal::entityTypeManager()->getStorage('contact_message'); /** @var ContentEntityStorageInterface $contact_message_storage */
    $contact_message = $contact_message_storage->create(['contact_form' => $contact_form_id]); /** @var MessageInterface $contact_message */

    $entity_form_builder = \Drupal::service('entity.form_builder'); /** @var EntityFormBuilderInterface $entity_form_builder */
    $form = $entity_form_builder->getForm($contact_message);
    $form['#cache']['contexts'][] = 'user.permissions';

    return $form;
  }

}
